/**
 * Read explanation on:
 * https://home.et.utwente.nl/slootenvanf/2019/11/28/object-counting-with-distance-sensors/
 * 
 * Ultrasonic sensor uses NewPing library: install via Tools > Manage Libraries... search "newping" and click Install
 * https://bitbucket.org/teckel12/arduino-new-ping/wiki/Home
 * 
 * Display uses U8g2lib: install via Tools > Manage Libraries... search "u8g2" and click Install
 * https://github.com/olikraus/u8g2
 */

#include <NewPing.h>
#include <U8g2lib.h>

#define TRIGGER_PIN  4  // Arduino pin tied to trigger pin on the ultrasonic sensor.
#define ECHO_PIN     5  // Arduino pin tied to echo pin on the ultrasonic sensor.
#define MAX_DISTANCE 300 // Maximum distance we want to ping for (in centimeters). Maximum sensor distance is rated at 400-500cm.

NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE); // NewPing setup of pins and maximum distance.

int height = 40; // you will have to calibrate this
int count = 0;
int product = 0;

// display setup:
// software i2c:
//U8X8_SSD1306_128X64_NONAME_SW_I2C u8x8(7, 8, U8X8_PIN_NONE);
// hardware i2c:
U8X8_SSD1306_128X64_NONAME_HW_I2C display(U8X8_PIN_NONE);

char buf[10]; // text buffer; to be able to use draw2x2String to show the value

void setup() {
  Serial.begin(9600); // start serial port output, check for same speed at Serial Monitor

  // initialize display:
  display.begin();
  display.setPowerSave(0);

  display.setFont(u8x8_font_pxplusibmcgathin_f);
  display.drawString(0,0,"Product counter");
  display.draw2x2String(0,2,"Count: ");
  display.draw2x2String(0,5,"-");
}

void loop() {
  unsigned int distance = sonar.convert_cm(sonar.ping_median(5)); // read distance from ultrasonic sensor (using median of 5 iterations)
  Serial.print("Dist.: ");
  Serial.println(distance);
  
  // display value on the display:
  display.drawString(0,1,"Dist.: ");
  display.setCursor(7, 1); display.print("     "); // wipe space to avoid remains of previous numbers
  display.setCursor(7, 1); display.print(distance);

  if (distance<3) { delay(200); return; } // skip error readings

  if (distance>=height) { // nothing seen
    if (product==1) { // change from 1>0: product no longer in sight confirmed
        count++;
        // displaying variable speed in a large (2x2) font:
        snprintf(buf, 10, "%4d", count); // put value of count into string buffer; %2d means we have two decimal places (because our value is always between 0 - 30)
        display.draw2x2String(0,5,buf);
        display.drawString(0,7,"-      ");
    }
    product--;
    if (product<0) product=0;
  }
  else { // product in sight
    if (product==2) { // change from 2>3: product presence confirmed
        display.drawString(0,7,"Product");
    }
    product++;
    if (product>3) product=3;
  }

  delay(50); // wait
}
